#!/bin/bash

# $Revision: 567 $ $Date: 2009-09-18 10:35:05 +0200 (Fri, 18 Sep 2009) $ $LastChangedBy: dream $

#
# Build script for the DREAM Toolbox (requires gcc).
#

ARCH=`uname -m`;
SYSTEM=`uname -s`;

if [ $ARCH = "i686" ] || [ $ARCH = "i386" ] ; then
    ARCH_FLAGS=

    if [ $SYSTEM = "Linux" ] ; then
	MEX_EXT=mexglx
    fi    
    
    # MacOS X (Intel Mac)
    if [ $SYSTEM = "Darwin" ] ; then
	MEX_EXT=mexmaci
	ARCH_FLAGS="-m64 -DUSE_8_BYTE_INTS"
    fi
    
elif [ $ARCH = "x86_64" ] ; then
    ARCH_FLAGS="-m64 -DUSE_8_BYTE_INTS"
    MEX_EXT=mexa64
fi

# Build the detectCPU app (only works on
# machines with the cpuid instruction).
if [ $ARCH = "i686" ] || [ $ARCH = "x86_64" ] || [ $ARCH = "i386" ] ; then
    cd detectCPU; make clean; make ; cd ..
    CPU=`./detectCPU/detectCPU -model`;
else
    CPU="UNKNOWN"
fi

#
# CFLAGS/CXXFLAGS
#
if [ $CPU = "COPPERMINE" ] ; then
    CFLAGS="-Wall -Wimplicit -pipe -O3 -fomit-frame-pointer \
	-funroll-loops -march=pentium3 -mtune=pentium3 -mfpmath=sse -mieee-fp"
elif [ $CPU = "NORTHWOOD" ] ; then
    CFLAGS="-Wall -Wimplicit -pipe -O1 -march=pentium4 -mtune=pentium4 \
	-funroll-loops -fomit-frame-pointer -mfpmath=sse -mieee-fp"
elif [ $CPU = "PRESCOTT" ] ; then	
	CFLAGS="-Wall -Wimplicit -pipe -O3 -march=prescott -mtune=prescott \
		-funroll-loops -fomit-frame-pointer -mfpmath=sse -mieee-fp"
elif [ $CPU = "BANIAS" ] ; then
	CFLAGS="-Wall -pipe -O3 -march=pentium-m -mtune=pentium-m \
		-funroll-loops -fomit-frame-pointer -mfpmath=sse -mieee-fp"
elif [ $CPU = "OPTERON" ] ; then 
    CFLAGS="$ARCH_FLAGS  -Wall -Wimplicit -O3 -march=opteron -mtune=opteron \
	-funroll-loops -fomit-frame-pointer -mfpmath=sse -mieee-fp"
elif [ $CPU = "CORE2" ] ; then	
    CFLAGS="$ARCH_FLAGS -Wall -Wimplicit -O3 -march=core2 -mtune=core2 \
	-funroll-loops -fomit-frame-pointer -mfpmath=sse -mieee-fp"
elif [ $CPU = "PENRYN" ] ; then	
    CFLAGS="$ARCH_FLAGS -Wall -Wimplicit -O3 -march=core2 -mtune=core2 \
        -mfpmath=sse -msse2 -msse3 -mssse3 -msse4.1 -mieee-fp \
	-funroll-loops -fomit-frame-pointer"
else
    #CFLAGS="-Wall -Wimplicit -pipe -O3 -funroll-loops -fomit-frame-pointer -mieee-fp"
    CFLAGS="-Wall -Wimplicit -pipe -O3 -funroll-loops -fomit-frame-pointer"
fi

#
# Test if Matlab and Octave is installed.
#

type matlab > /dev/null 2>&1
if [ $? -eq 0 ] ; then 
    echo ""
    echo "* A Matlab installation is found."
    
    file /usr/local/matlab/extern/include/mex.h > /dev/null 2>&1
    if [ $? -eq 0 ] ; then 
	BUILD_MATLAB="TRUE"
    else
	echo ""
	echo "The file: /usr/local/matlab/extern/include/mex.h"
	echo "is not found! Is matlab installed in a non-standard location?"
	echo "  ln -s your_install_dir /usr/local/matlab may solve this" 
	echo "issue."
    fi
else
    echo ""
    echo "* Matlab is not found! Do you have matlab in your path?"
    BUILD_MATLAB="FALSE"
fi

type octave > /dev/null 2>&1
if [ $? -eq 0 ] ; then 
    echo ""
    echo "* You have Octave version `octave -version | grep version | awk '{ print $4 }'` installed."
    BUILD_OCTAVE="TRUE"
else
    echo ""
    echo "* Octave is not found!"
    BUILD_OCTAVE="FALSE"
fi

echo ""
echo "* A $CPU CPU has been detected on a $ARCH architecture."
echo "* on a $SYSTEM system."
echo ""
echo "* DREAM will be build with the following flags:"
echo "  CFLAGS/CXXFLAGS = $CFLAGS"
#echo "ARCH_FLAGS = $ARCH_FLAGS"
echo ""
echo "* The toolbox will be installed in: $PWD/arch/$CPU/dream/"
echo ""
echo "Build will start in 5 [s]"
sleep 1
echo "Build will start in 4 [s]"
sleep 1
echo "Build will start in 3 [s]"
sleep 1
echo "Build will start in 2 [s]"
sleep 1
echo "Build will start in 1 [s]"
sleep 1
echo ""

# Build the Make.inc file.

cat Make.pre > Make.inc

# Installation dirs.
echo " "  >> Make.inc
echo "INSTALL_DIR = arch/$CPU/dream/" >> Make.inc
echo " "  >> Make.inc

# Version of the DREAM Toolbox.
echo " "  >> Make.inc
echo "VER = `cat VERSION`" >> Make.inc
echo " "  >> Make.inc

# Compiler flags.
echo "CFLAGS = $CFLAGS" >> Make.inc
echo " "  >> Make.inc

# Set the mex extension. 
echo "# Platform dependent suffix of mex-files." >> Make.inc
echo "MEX_EXT = $MEX_EXT" >> Make.inc

if [ $BUILD_OCTAVE = "TRUE" ] ; then
    # Set Octave version.
    echo " "  >> Make.inc
    echo "OCTVER = `octave -version | grep version | awk '{ print $4 }'`" >> Make.inc
    echo " "  >> Make.inc
    
    # Set the Octave path.
    echo " "  >> Make.inc
    echo "OCT_PATH = `which octave | awk '{ sub("/bin/octave", ""); print }'`" >> Make.inc
    echo " "  >> Make.inc
fi

cat Make.post >> Make.inc

# Build the toolbox and documentation.

if [ $BUILD_MATLAB = "TRUE" ] ; then
    make -f Makefile matlab
fi

if [ $BUILD_OCTAVE = "TRUE" ] ; then
    make -f Makefile octave 
fi

if [ $BUILD_MATLAB = "FALSE"  ] && [ $BUILD_OCTAVE = "FALSE"  ] ; then
    echo "Cannot build DREAM whithout a Matlab or Octave installation!"
    exit
fi

make -f Makefile doc
