%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Script for making the DREAM mex-files for Windows using the
% MinGW-w64/Gnumex tools without fftw support for the attenuation
% code. Note the MinGW-w64 compiler currently cannot  link to the 
% Matlab libs so this file is olny for future use! 
%
% Copyright (C) 2008,2009 Fredrik Lingvall
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% $Revision: 601 $ $Date: 2009-11-08 16:45:24 +0100 (Sun, 08 Nov 2009) $ $LastChangedBy: dream $

disp(' ');
disp('Note you need to install pthreadGC2.dll and libfftw3-3.dll ');
disp('in C:\WINDOWS\System32\ before you use these mex-files!')
disp(' ');
pause(5);

%
% Flags.
%

TOOLBOX = 1;
MISC = 1;
EXTRA = 1; 
INSTALL = 1;

if TOOLBOX

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  % Build the MEX-files using MinGW-w64 gcc and gnumex tools (Windows)
  %
  %	     http://www.mingw.org
  %	     http://www.mrc-cbu.cam.ac.uk/Imaging/gnumex20.html
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  if (strcmp(computer,'PCWIN') | strcmp(computer,'PCWIN64') )    
    cd src
    
    %
    % Single transducer functions.
    %
    fprintf('Building the DREAM toolbox MEX-functions using (MinGW-w64) gcc:\n\n');
    
    fprintf('Compiling dreamline...');
    mex -v -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamline.c dreamline.c att.c fft.c dream_error.c -output dreamline
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamrect.c dreamrect.c att.c fft.c dream_error.c -output dreamrect
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect_f...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamrect_f.c dreamrect_f.c att.c fft.c dream_error.c -output dreamrect_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamcirc.c dreamcirc.c att.c fft.c dream_error.c -output dreamcirc
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc_f...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamcirc_f.c dreamcirc_f.c att.c fft.c dream_error.c -output dreamcirc_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_f...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamcylind_f.c dreamcylind_f.c att.c fft.c dream_error.c -output dreamcylind_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_d...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamcylind_d.c dreamcylind_d.c att.c fft.c dream_error.c -output dreamcylind_d
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_f...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamsphere_f.c dreamsphere_f.c att.c fft.c dream_error.c -output dreamsphere_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_d...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dreamsphere_d.c dreamsphere_d.c att.c fft.c dream_error.c -output  dreamsphere_d
    fprintf('done!\n');
    
    % Array functions.
    
    fprintf('Compiling dream_arr_circ...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dream_arr_circ.c dream_arr_circ.c ...
        arr_functions.c att.c fft.c  dream_error.c -output dream_arr_circ
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_rect...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
        mex_dream_arr_rect.c dream_arr_rect.c ...
        arr_functions.c  att.c fft.c dream_error.c -output dream_arr_rect
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_f...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_dream_arr_cylind_f.c dream_arr_cylind_f.c ...
        dream_error.c arr_functions.c att.c fft.c -output dream_arr_cylind_f
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_d...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_dream_arr_cylind_d.c dream_arr_cylind_d.c ...
        arr_functions.c att.c fft.c  dream_error.c -output dream_arr_cylind_d
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_annu...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_dream_arr_annu.c dream_arr_annu.c ...
        att.c fft.c  dream_error.c -output dream_arr_annu
    
    cd ../
    
    fprintf('\nBuild of (serial) DREAM toolbox functions done!\n');
  end
  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  % Parallel functions (for Windows only). For Linux use the Makefile
  % (see README.pdf).
  %
  %
  % Note: MinGW-w64 gcc tools, win32 Pthreads lib, and gnumex tools are
  % needed for these functions:
  %
  %	     http://www.mingw.org
  %	     http://sources.redhat.com/pthreads-win32/
  %	     http://www.mrc-cbu.cam.ac.uk/Imaging/gnumex20.html
  %
  % Put threadGC.dll in a windows system dir and the parellel mex-functions
  % should work.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  if (strcmp(computer,'PCWIN') || strcmp(computer,'PCWIN64') )    
    
    fprintf('\nBuilding the parallel DREAM toolbox MEX-functions using (MinGW-w64) gcc:\n\n');
    
    cd src   
    
    fprintf('Compiling dreamline_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamline_p.c dreamline.c att.c fft.c dream_error.c -output dreamline_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamrect_p.c dreamrect.c att.c fft.c dream_error.c -output dreamrect_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect_f_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamrect_f_p.c dreamrect_f.c att.c fft.c dream_error.c -output dreamrect_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamcirc_p.c dreamcirc.c att.c fft.c dream_error.c -output dreamcirc_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc_f_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamcirc_f_p.c dreamcirc_f.c att.c fft.c dream_error.c  -output dreamcirc_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_f_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamcylind_f_p.c dreamcylind_f.c att.c fft.c dream_error.c -output dreamcylind_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_d_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamcylind_d_p.c dreamcylind_d.c att.c fft.c dream_error.c -output dreamcylind_d_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_f_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamsphere_f_p.c dreamsphere_f.c att.c fft.c dream_error.c -output dreamsphere_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_d_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dreamsphere_d_p.c dreamsphere_d.c att.c fft.c dream_error.c -output  dreamsphere_d_p
    fprintf('done!\n');
    
    % Array functions.
    
    fprintf('Compiling dream_arr_circ_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dream_arr_circ_p.c dream_arr_circ.c ...
        arr_functions.c att.c fft.c  dream_error.c -output dream_arr_circ_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_rect_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dream_arr_rect_p.c dream_arr_rect.c ...
        arr_functions.c  att.c fft.c dream_error.c -output dream_arr_rect_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_f_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dream_arr_cylind_f_p.c dream_arr_cylind_f.c ...
        dream_error.c arr_functions.c att.c fft.c  -output dream_arr_cylind_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_d_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dream_arr_cylind_d_p.c dream_arr_cylind_d.c ...
        arr_functions.c att.c fft.c  dream_error.c -output dream_arr_cylind_d_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_annu_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_dream_arr_annu_p.c dream_arr_annu.c ...
        att.c fft.c  dream_error.c  -output dream_arr_annu_p
    fprintf('done!\n');
    
    cd ../
    
    fprintf('\nBuild of parallel DREAM toolbox functions done!\n');  
  end
  
end

if MISC
  
  if (strcmp(computer,'PCWIN') | strcmp(computer,'PCWIN64') )

    %
    % Misc functions.
    %
    
    fprintf('\nBuilding the Misc DREAM toolbox MEX-functions using (MinGW-w64) gcc:\n\n');
    
    cd src
    
    fprintf('Compiling dream_apodwin...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_dream_apodwin.c arr_functions.c ...
        dream_error.c -output dream_apodwin
    fprintf('done!\n');
    
    fprintf('Compiling dream_att...');
    mex -I../include  -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_dream_att.c att.c fft.c dream_error.c ...
        -output dream_att
    fprintf('done!\n');
    
    cd ../
    
    %
    
    cd extra_src
    
    fprintf('Compiling conv_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_conv_p.c dream_error.c -output conv_p
    fprintf('done!\n');
    
    fprintf('Compiling fftconv_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_fftw_mingw_64.bat ...
         mex_fftconv_p.c dream_error.c -output fftconv_p
    fprintf('done!\n');
    
    fprintf('Compiling sum_fftconv_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_fftw_mingw_64.bat ...
         mex_sum_fftconv_p.c dream_error.c -output sum_fftconv_p
    fprintf('done!\n');
    
    fprintf('Compiling copy_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64.bat ...
         mex_copy_p.c dream_error.c -output copy_p
    fprintf('done!\n');
    
    cd ../
    
  end
end

if EXTRA

  if (strcmp(computer,'PCWIN') || strcmp(computer,'PCWIN64') )
    
    cd extra_src
    
    fprintf('\nBuilding the Extra DREAM toolbox MEX-functions using (MinGW-w64) gcc:\n\n');

    fprintf('Compiling das...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_das.c das.c dream_error.c -output das
    fprintf('done!\n');
    
    fprintf('Compiling das_arr...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_das_arr.c  das_arr.c arr_functions.c dream_error.c -output das_arr
    fprintf('done!\n');
    
    %
    % Analytic methods.
    %
    
    fprintf('Compiling circ_sir...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_circ_sir.c circ_sir.c dream_error.c -output circ_sir
    fprintf('done!\n');
    
    fprintf('Compiling scirc_sir...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_scirc_sir.c scirc_sir.c dream_error.c -output scirc_sir
    fprintf('done!\n');
    
    fprintf('Compiling rect_sir...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_rect_sir.c rect_sir.c dream_error.c -output rect_sir
    fprintf('done!\n');
    
    %
    % SAFT
    %

    fprintf('Compiling saft...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_mingw_64.bat ...
         mex_saft.c dream_error.c -output saft
    fprintf('done!\n');
    
    fprintf('Compiling saft_p...');
    mex -I../include -I../windows/include -f ../windows/mexopts/mexopts_pthread_mingw_64bat ...
         mex_saft_p.c dream_error.c -output saft_p
    fprintf('done!\n');
    
    
    cd ../
    
    fprintf('\nBuild of Extra DREAM toolbox functions done!\n');
  end  
end

if INSTALL
  % Install
  cd src
  system('mkdir ..\arch\x86_64')
  system('mkdir ..\arch\x86_64\dream')
  system('copy *.mexw64 ..\arch\x86_64\dream\')
  cd ../

  cd extra_src
  system('copy *.mexw64 ..\arch\x86_64\dream\')
  cd ../
  fprintf('\nThe mex-files are installed in: %s/arch/x86_64/dream \n',pwd)
end

