%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Script for making the DREAM (64-bit) mex-files for Windows.
%
% The mex-files are currently compiled without fftw support for the 
% attenuation code since the MSVC 2008 Express C compiler don't
% have support for the complex data type [complex.h is missing] 
% which is needed when using the fftw lib.
%
% Copyright (C) 2008,2009 Fredrik Lingvall
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% $Revision: 617 $ $Date: 2009-11-08 21:54:10 +0100 (Sun, 08 Nov 2009) $ $LastChangedBy: dream $

disp(' ');
disp('Note you need to install 64-bit versions of pthreadGC2.dll and libfftw3-3.dll ');
disp('in C:\WINDOWS\System32\ before you use these mex-files!')
disp(' ');
pause(5);

%
% Flags.
%

TOOLBOX = 1;
MISC = 1;
EXTRA = 1;
INSTALL = 1;

if TOOLBOX

  if (strcmp(computer,'PCWIN') || strcmp(computer,'PCWIN64'))
    
    cd src
    
    %
    % Single transducer functions.
    %
    
    fprintf('Building the DREAM toolbox MEX-functions:\n\n');
    
    fprintf('Compiling dreamline...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        mex_dreamline.c dreamline.c att.c fft.c dream_error.c -output dreamline
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamrect.c dreamrect.c att.c fft.c dream_error.c -output dreamrect
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect_f...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamrect_f.c dreamrect_f.c att.c fft.c dream_error.c -output dreamrect_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamcirc.c dreamcirc.c att.c fft.c dream_error.c -output dreamcirc
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc_f...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamcirc_f.c dreamcirc_f.c att.c fft.c dream_error.c  -output dreamcirc_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_f...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamcylind_f.c dreamcylind_f.c att.c fft.c dream_error.c -output dreamcylind_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_d...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamcylind_d.c dreamcylind_d.c att.c fft.c dream_error.c -output dreamcylind_d
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_f...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamsphere_f.c dreamsphere_f.c att.c fft.c dream_error.c -output dreamsphere_f
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_d...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dreamsphere_d.c dreamsphere_d.c att.c fft.c dream_error.c -output dreamsphere_d
    fprintf('done!\n');
    
    % Array functions.
    
    fprintf('Compiling dream_arr_circ...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dream_arr_circ.c dream_arr_circ.c ...
        arr_functions.c att.c fft.c  dream_error.c -output dream_arr_circ
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_rect...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
        mex_dream_arr_rect.c dream_arr_rect.c ...
        arr_functions.c  att.c fft.c dream_error.c -output dream_arr_rect
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_f...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
         mex_dream_arr_cylind_f.c dream_arr_cylind_f.c ...
        dream_error.c arr_functions.c att.c fft.c  -output dream_arr_cylind_f
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_d...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
         mex_dream_arr_cylind_d.c dream_arr_cylind_d.c ...
        arr_functions.c att.c fft.c  dream_error.c  -output dream_arr_cylind_d
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_annu...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
         mex_dream_arr_annu.c dream_arr_annu.c ...
        att.c fft.c  dream_error.c  -output dream_arr_annu
    
    cd ../
    
    fprintf('\nBuild of (serial) DREAM toolbox functions done!\n');
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  % Parallel functions (for Windows only). 
  % 
  % Note: the win32 Pthreads lib is needed for these functions (see the
  % DREAM user manulal).
  %	     
  %
  % Put threadGC2.dll in a Windows system dir and the parellel mex-functions
  % should work.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  if (strcmp(computer,'PCWIN') || strcmp(computer,'PCWIN64'))
    
    fprintf('\nBuilding the parallel DREAM toolbox MEX-functions:\n\n');
    
    cd src
    
    fprintf('Compiling dreamline_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat ...
        -I../include -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamline_p.c dreamline.c att.c fft.c dream_error.c -output dreamline_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamrect_p.c dreamrect.c att.c fft.c dream_error.c -output dreamrect_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamrect_f_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamrect_f_p.c dreamrect_f.c att.c fft.c dream_error.c -output dreamrect_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat ...
        -I../include -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamcirc_p.c dreamcirc.c att.c fft.c dream_error.c -output dreamcirc_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcirc_f_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamcirc_f_p.c dreamcirc_f.c att.c fft.c dream_error.c  -output dreamcirc_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_f_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamcylind_f_p.c dreamcylind_f.c att.c fft.c dream_error.c -output dreamcylind_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamcylind_d_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamcylind_d_p.c dreamcylind_d.c att.c fft.c dream_error.c -output dreamcylind_d_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_f_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamsphere_f_p.c dreamsphere_f.c att.c fft.c dream_error.c   -output dreamsphere_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dreamsphere_d_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dreamsphere_d_p.c dreamsphere_d.c att.c fft.c dream_error.c   -output  dreamsphere_d_p
    fprintf('done!\n');
    
    % Array functions.
    
    fprintf('Compiling dream_arr_circ_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dream_arr_circ_p.c dream_arr_circ.c ...
        arr_functions.c att.c fft.c  dream_error.c -output dream_arr_circ_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_rect_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dream_arr_rect_p.c dream_arr_rect.c ...
        arr_functions.c  att.c fft.c dream_error.c -output dream_arr_rect_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_f_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dream_arr_cylind_f_p.c dream_arr_cylind_f.c ...
        dream_error.c arr_functions.c att.c fft.c  -output dream_arr_cylind_f_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_cylind_d_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dream_arr_cylind_d_p.c dream_arr_cylind_d.c ...
        arr_functions.c att.c fft.c  dream_error.c  -output dream_arr_cylind_d_p
    fprintf('done!\n');
    
    fprintf('Compiling dream_arr_annu_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_dream_arr_annu_p.c dream_arr_annu.c ...
        att.c fft.c  dream_error.c  -output dream_arr_annu_p
    fprintf('done!\n');
    
    cd ../
    
    fprintf('\nBuild of parallel DREAM toolbox functions done!\n');  
  end
  
end

if MISC
  
  if (strcmp(computer,'PCWIN') || strcmp(computer,'PCWIN64'))
      
    %
    % Misc functions.
    %
    
    fprintf('\nBuilding the Misc DREAM toolbox MEX-functions:\n\n');
    
    cd src
    
    fprintf('Compiling dream_apodwin...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
         mex_dream_apodwin.c arr_functions.c ...
        dream_error.c -output dream_apodwin
    fprintf('done!\n');
    
    fprintf('Compiling dream_att...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include  ...
         mex_dream_att.c att.c fft.c dream_error.c ...
        -output dream_att
    fprintf('done!\n');
    
    cd ../
    
    %
    cd extra_src
    
    fprintf('Compiling conv_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_conv_p.c dream_error.c -output conv_p
    fprintf('done!\n');
    
    % Must compile this one with C++ since the MS C compiler cannot
    % handle complex doubles.
    fprintf('Compiling fftconv_p...');
    mex -DMSVC2008_FFTW_64_BUG -DUSE_8_BYTE_INTS ...
        -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 ...
    -lpthreadGC2 -llibfftw3-3 mex_fftconv_p_msvc.cc dream_error.c -output fftconv_p
    fprintf('done!\n');
    
    % Must compile this one with C++ since the MS C compiler cannot
    % handle complex doubles.
    fprintf('Compiling sum_fftconv_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 -llibfftw3-3 ...
        mex_sum_fftconv_p_msvc.cc dream_error.c -output sum_fftconv_p
    fprintf('done!\n');
    
    fprintf('Compiling copy_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_copy_p.c dream_error.c -output copy_p
    fprintf('done!\n');
    
    cd ../
    
  end
end

if EXTRA

  if (strcmp(computer,'PCWIN') || strcmp(computer,'PCWIN64') )
    
    cd extra_src
    
    fprintf('\nBuilding the Extra DREAM toolbox MEX-functions:\n\n');

    fprintf('Compiling das...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 ...
        mex_das.c das.c dream_error.c -output das
    fprintf('done!\n');
    
    fprintf('Compiling das_arr...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 ...
        mex_das_arr.c  das_arr.c arr_functions.c dream_error.c -output das_arr
    fprintf('done!\n');
    
    %
    % Analytic methods.
    %
    
    fprintf('Compiling circ_sir...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 ...
        mex_circ_sir.c circ_sir.c dream_error.c -output circ_sir
    fprintf('done!\n');
    
    fprintf('Compiling scirc_sir...');
    mex -DUSE_8_BYTE_INTS -I../include -f ../windows/mexopts/mexopts_msvc_64.bat ...
        -I../windows/dll_64 ...
        mex_scirc_sir.c scirc_sir.c dream_error.c -output scirc_sir
    fprintf('done!\n');
    
    fprintf('Compiling rect_sir...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 ...
        mex_rect_sir.c rect_sir.c dream_error.c -output rect_sir
    fprintf('done!\n');
    
    %
    % SAFT
    %

    fprintf('Compiling saft...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat -I../include ...
        -I../windows/dll_64 ...
        mex_saft.c dream_error.c -output saft
    fprintf('done!\n');
    
    fprintf('Compiling saft_p...');
    mex -DUSE_8_BYTE_INTS -f ../windows/mexopts/mexopts_msvc_64.bat ...
        -I../include -I../windows/dll_64 -L../windows/dll_64 -lpthreadGC2 ...
        mex_saft_p.c dream_error.c -output saft_p
    fprintf('done!\n');
    
    cd ../
    
    fprintf('\nBuild of Extra DREAM toolbox functions done!\n');
  end  
end

if INSTALL
  % Install
  cd src
  system('mkdir ..\arch\x86_64');
  system('mkdir ..\arch\x86_64\windows');
  system('copy *.mexw64 ..\arch\x86_64\windows\');
  cd ../

  cd extra_src
  system('copy *.mexw64 ..\arch\x86_64\windows\');
  cd ../
  fprintf('\nThe mex-files are installed in: %s/arch/x86_64/windows \n',pwd)
end
