/***
*
* Copyright (C) 2003,2006,2007,2008,2009 Fredrik Lingvall
*
* This file is part of the DREAM Toolbox.
*
* The DREAM Toolbox is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2, or (at your option) any
* later version.
*
* The DREAM Toolbox is distributed in the hope that it will be useful, but 
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License
* along with the DREAM Toolbox; see the file COPYING.  If not, write to the 
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
* 02110-1301, USA.
*
***/

// $Revision: 565 $ $Date: 2009-09-17 22:24:06 +0200 (Thu, 17 Sep 2009) $ $LastChangedBy: dream $

#include <math.h>
#include <stdio.h>
#include "das.h"
#include "dream_error.h"

#if defined(_MSC_VER) || defined(__LCC__)
#include "msvc_rint.h"
#endif

/***
 *
 *  Delay-and-sum response.
 *
 ***/

/***
 *
 *  das - Delay-and-sum.
 *
 ***/

int das(double xo, double yo, double zo,
	double dt, dream_idx_type  nt, double delay,
	double cp, double *h, int err_level)
{
  dream_idx_type i,it;
  double t,tt;
  double pi, ri;
  double qan;
  //double xs, ys, zs;
  int err = NONE;

  pi = atan((double) 1.0) * 4.0;

  //xs = 0.0;
  //zs = 0.0;
  //ys = 0.0; // Center transducer at origo.

  for (i = 0; i < nt; i++) {
    h[i] = (double) 0.0 ;
  }
  
  //modri(xo, yo, zo, xs, ys, zs, &ri);
  //ri = sqrt((xo-xs)*(xo-xs) + (yo-ys)*(yo-ys) + (zo-zs)*(zo-zs));
  ri = sqrt( xo*xo + yo*yo + zo*zo );

  t = ri * 1000.0/cp;
  tt = t - delay;
  qan = tt / dt;
  it = 0;
  it = (dream_idx_type) rint(qan);

  // Check if index is out of bounds.
  if ((it < nt) && (it >= 0)) {
    h[it] += 1.0;
  }
  else  {
    if  (it >= 0)
      err = dream_out_of_bounds_err("Delay out of bounds",it-nt+1,err_level);
    else
      err = dream_out_of_bounds_err("Delay out of bounds",it,err_level);

    if ( (err_level == PARALLEL_STOP) || (err_level == STOP) )    
      return err; // Bail out.
  }

  return err;
} /* das */

