/***
*
* Copyright (C) 2004,2006,2007,2008,2009 Fredrik Lingvall
*
* This file is part of the DREAM Toolbox.
*
* The DREAM Toolbox is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2, or (at your option) any
* later version.
*
* The DREAM Toolbox is distributed in the hope that it will be useful, but 
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License
* along with the DREAM Toolbox; see the file COPYING.  If not, write to the 
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
* 02110-1301, USA.
*
***/

// $Revision: 565 $ $Date: 2009-09-17 22:24:06 +0200 (Thu, 17 Sep 2009) $ $LastChangedBy: dream $

#include <string.h>
#include "mex.h"
#include "rect_sir.h"
#include "dream_error.h"

/***
 *
 * Matlab (MEX) gateway function for RECT_SIR.
 *
 ***/

void  mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  double *ro,*geom_par,*s_par,*m_par;
  dream_idx_type nt,no,n;
  double xo, yo, zo, a, b, dt;
  double *delay,v,cp;     
  double *h;
  
  // Check for proper number of arguments
    
  if (nrhs != 5) {
    dream_err_msg("rect_sir requires 5 input arguments!");
  }
  else
    if (nlhs > 1) {
      dream_err_msg("Too many output arguments for rect_sir!");
    }
  
  //
  // Observation point.
  //
  
 // Check that arg (number of observation points) x 3 matrix
  if (!mxGetN(prhs[0])==3)
    dream_err_msg("Argument 1 must be a (number of observation points) x 3 matrix!");
  
  no = mxGetM(prhs[0]); // Number of observation points.
  ro = mxGetPr(prhs[0]);


  //
  // Transducer geometry
  //

   // Check that arg 2 is a scalar.
  if (!((mxGetM(prhs[1])==2 && mxGetN(prhs[1])==1) || (mxGetM(prhs[1])==1 && mxGetN(prhs[1])==2)))
    dream_err_msg("Argument 2 must be a two element vector!");
  
  geom_par = mxGetPr(prhs[1]);
  a  = geom_par[0];		// x-dim of the transducer.
  b  = geom_par[1];		// y-dim of the transducer.

  //
  // Temporal and spatial sampling parameters.
  //
  
  // Check that arg 3 is a 2 element vector
  if (!((mxGetM(prhs[2])==2 && mxGetN(prhs[2])==1) || (mxGetM(prhs[2])==1 && mxGetN(prhs[2])==2)))
    dream_err_msg("Argument 3 must be a vector of length 2!");
  
  s_par = mxGetPr(prhs[2]);
  dt    = s_par[0];		// Temporal discretization size (= 1/sampling freq).
  nt    = (dream_idx_type) s_par[1];	// Length of SIR.
  
  //
  // Start point of impulse response vector ([us]).
  //

  // Check that arg 4 is a scalar.
  if ( (mxGetM(prhs[3]) * mxGetN(prhs[3]) !=1) && ((mxGetM(prhs[3]) * mxGetN(prhs[3])) != no))
    dream_err_msg("Argument 4 must be a scalar or a vector with a length equal to the number of observation points!");
  
  delay = mxGetPr(prhs[3]);
  
  //
  // Material parameters
  //
  
  // Check that arg 5 is a 2 element vector.
  if (!((mxGetM(prhs[4])==2 && mxGetN(prhs[4])==1) || (mxGetM(prhs[4])==1 && mxGetN(prhs[4])==2)))
    dream_err_msg("Argument 5 must be a vector of length 2!");
  
  m_par = mxGetPr(prhs[4]);
  v     = m_par[0]; // Normal velocity of transducer surface.
  cp    = m_par[1]; // Sound speed.
  
  //  
  // Create an output matrix for the impulse response(s).
  //
  plhs[0] = mxCreateDoubleMatrix(nt,no,mxREAL);
  h = mxGetPr(plhs[0]);
  
  // Call the analytic SIR subroutine.
  if (mxGetM(prhs[3]) * mxGetN(prhs[3]) == 1) {
    for (n=0; n<no; n++) {
      xo = ro[n]; 
      yo = ro[n+1*no]; 
      zo = ro[n+2*no]; 
      
      rect_sir(xo,yo,zo,a,b,dt,nt,delay[0],v,cp,&h[n*nt]);
    }
  } else {
    for (n=0; n<no; n++) {
      xo = ro[n]; 
      yo = ro[n+1*no]; 
      zo = ro[n+2*no]; 
      
      rect_sir(xo,yo,zo,a,b,dt,nt,delay[n],v,cp,&h[n*nt]);
    }
  }
    
  return;
}
      
