/***
*
* Copyright (C) 2002,2003,2006,2008,2009 Fredrik Lingvall
*
* This file is part of the DREAM Toolbox.
*
* The DREAM Toolbox is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2, or (at your option) any
* later version.
*
* The DREAM Toolbox is distributed in the hope that it will be useful, but 
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License
* along with the DREAM Toolbox; see the file COPYING.  If not, write to the 
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
* 02110-1301, USA.
*
***/

// $Revision: 565 $ $Date: 2009-09-17 22:24:06 +0200 (Thu, 17 Sep 2009) $ $LastChangedBy: dream $

#include <string.h>
#include "mex.h"
#include "arr_functions.h"
#include "dream_error.h"

/***
 *
 * Matlab (mex) gateway function for dream_apodwin (for the weighting functions in 
 * arr_functions.c)
 *
 ***/

void  mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  char   apod_met[40];
  int    buflen;
  int    iweight=0,iapo=0,i,isize=0, set = FALSE;
  double *apod=NULL,weight,xs,ys,ramax,param;
  double *yr; 


  // Check for proper number of arguments
  if (nrhs != 3) {
    dream_err_msg("dream_apodwin requires three input arguments!");
  }
  else
    if (nlhs > 1) {
      dream_err_msg("dream_apodwin requires one output argument!");
    }

  
  buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0]) * sizeof(mxChar)) + 1;
  mxGetString(prhs[0],apod_met,buflen);
  
  iweight = 1;			// default off. 

  set = FALSE;  
  if (!strcmp(apod_met,"off")) {
    iweight = 1;
    set = TRUE; 
  }
  
  if (!strcmp(apod_met,"ud")) {
    iweight = 2; 
    iapo = 0; 
    dream_err_msg(" 'ud'- (user defined) meaningless for this function!");
  }
  
  if (!strcmp(apod_met,"triangle")) {
    iweight = 2; 
    iapo = 1; 
    set = TRUE; 
  }
  
  if (!strcmp(apod_met,"gauss")) {
    iweight = 2; 
    iapo = 2; 
    set = TRUE; 
  }
  
  if (!strcmp(apod_met,"raised")) {
    iweight = 2; 
    iapo = 3; 
    set = TRUE; 
  }
  
  if (!strcmp(apod_met,"simply")) {
    iweight = 2; 
    iapo = 4; 
    set = TRUE; 
  }
  
  if (!strcmp(apod_met,"clamped")) {
    iweight = 2; 
    iapo = 5; 
    set = TRUE; 
  }

  if (set == FALSE)
    dream_err_msg("Unknown apodization level!");


  //
  // Apodization.
  //
  
  // iapo = 0 - user defined, 1 traingle, 2 Gauss, 3 raised cosine, 4 simply supported, 5 clamped. 
  
  if (!mxIsChar(prhs[0]))
    dream_err_msg("Argument 1 must be a string");
  
  if (mxGetM(prhs[1]) * mxGetN(prhs[1]) !=1)
    dream_err_msg("Argument 2 must be a scalar!");  
  
  isize = (int) mxGetScalar(prhs[1]);
  
  //if (!mxIsInt32(prhs[1]) || isize < 0)
  if (isize < 0)
    dream_err_msg("Argument 2 must be a positive integer!");  

  //
  // param - Parameter used for raised cos and Gaussian apodization functions.
  //

  if (mxGetM(prhs[2]) * mxGetN(prhs[2]) !=1)
    dream_err_msg("Argument 3 must be a scalar!");  
  

  param = mxGetScalar(prhs[2]);
  
  // Create a matrix for return arguments
  plhs[0] = mxCreateDoubleMatrix(isize,1,mxREAL);
  yr = mxGetPr(plhs[0]);
  //yi = mxGetPi(plhs[0]);

  ramax = 1;

  ys = 0;
  if (iweight != 1) {	
    for (i=0; i<isize; i++) { 
      xs = 2*ramax * (0.5 - ((double) i / (double) isize));
      weighting(iweight,iapo,i,apod,&weight,xs,ys,ramax,param,isize);
      yr[i] = weight;
    }
  }
}
      
