/***
*
* Copyright (C) 2006,2007,2008,2009 Fredrik Lingvall
*
* This file is part of the DREAM Toolbox.
*
* The DREAM Toolbox is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2, or (at your option) any
* later version.
*
* The DREAM Toolbox is distributed in the hope that it will be useful, but 
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License
* along with the DREAM Toolbox; see the file COPYING.  If not, write to the 
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
* 02110-1301, USA.
*
***/

// $Revision: 565 $ $Date: 2009-09-17 22:24:06 +0200 (Thu, 17 Sep 2009) $ $LastChangedBy: dream $

#include <string.h>
#include <stdio.h>
#include <signal.h>
#include "dream_arr_cylind_d.h"
#include "dream_error.h"

#ifdef USE_FFTW
#include "att.h"
#endif

//
// Octave headers.
//

#include <octave/oct.h>

#include <octave/config.h>

#include <iostream>
using namespace std;

#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/oct-obj.h>
#include <octave/pager.h>
#include <octave/symtab.h>
#include <octave/variables.h>

//
// Macros
//

#define mxGetM(N)   args(N).matrix_value().rows()
#define mxGetN(N)   args(N).matrix_value().cols()
#define mxIsChar(N) args(N).is_string()

//
// Globals
//

int running;

//
// Function prototypes.
//

void sighandler(int signum);
void sig_abrt_handler(int signum);
void sig_keyint_handler(int signum);

//
// typedef:s
//

typedef void (*sighandler_t)(int);

/***
 *
 * Signal handlers.
 *
 ***/

void sighandler(int signum) {
  //printf("Caught signal SIGTERM.\n");
  running = FALSE;
}

void sig_abrt_handler(int signum) {
  //printf("Caught signal SIGABRT.\n");
}

void sig_keyint_handler(int signum) {
  //printf("Caught signal SIGINT.\n");
}


/***
 *
 * Octave (OCT) gateway function for dream_arr_cylind_d
 *
 ***/

DEFUN_DLD (dream_arr_cylind_d, args, nlhs,
	   "-*- texinfo -*-\n\
@deftypefn {Loadable Function} {} [H,err] = dream_arr_cylind_d(Ro,geom_par,G,s_par,delay,m_par,foc_met,...\n\
                 focal,steer_met,steer_par,apod_met,apod,win_par,err_level)\n\
\n\
DREAM_ARR_CYLIND_D - Computes the spatial impulse response\n\
for an array with cylindrical convex (defocused) elements.\n\
\n\
Observation point(s) ([mm]):\n\
\n\
@table @code\n\
@item Ro\n\
An N x 3 matrix, Ro = [xo1 yo1 zo2; xo2 yo2 zo2; ... xoN yoN zoN]; where N is the number of observation points.\n\
@end table\n\
\n\
Geometrical parameters: geom_par = [a b R];\n\
\n\
@table @code\n\
@item a\n\
Element width in x-direction [mm].\n\
@item b\n\
Element width in y-direction [mm].\n\
@item R\n\
Curvature radius (y-direction) [mm].\n\
@end table\n\
\n\
Array grid parameter:\n\
\n\
@table @code\n\
@item G\n\
An L x 3 grid function matrix. Column 1 contain the x-postions \n\
of the elements, column 2 the y-positions, and column 3\n\
the z-positions, and where L is the number of elements.\n\
@end table\n\
\n\
@table @code\n\
@item dx\n\
Spatial x-direction discretization size [mm] .\n\
@item dy\n\
Spatial y-direction discretization size [mm].\n\
@item dt\n\
Temporal discretization period (= 1/sampling freq) [us].\n\
@item  nt\n\
Length of impulse response vector.\n\
@end table\n\
\n\
Start point of SIR:\n\
\n\
@table @code\n\
@item  delay\n\
Scalar delay for all observation points or a vector with individual delays for each observation point [us].\n\
@end table\n\
\n\
Material parameters: m_par = [v cp alfa];\n\
\n\
@table @code\n\
@item v\n\
Normal velocity [m/s].\n\
@item cp\n\
Sound velocity [m/s].\n\
@item alfa\n\
Attenuation coefficient [dB/(cm MHz)] .\n\
\n\
@end table\n\
\n\
Focusing parameters: foc_met and focal:\n\
\n\
@table @code\n\
@item foc_met\n\
Focusing method, options are: 'off', 'x', 'y', 'xy', 'x+y', and 'ud'.\n\
@item  focal\n\
Focal distance [mm]. If foc_met = 'ud' (user defined) then focal is a vector of focusing delays.\n\
@end table\n\
\n\
Beam steering parameters: steer_met and steer_par:\n\
\n\
@table @code\n\
@item steer_met\n\
Beam steering method, options are: 'off', 'x', 'y', and 'xy'.\n\
@item  steer_par =  [theta phi];\n\
theta [deg] is the x-direction steer angle and \n\
phi [deg] is the y-direction steer angle.\n\
@end table\n\
\n\
@table @code\n\
\n\
@item 'off'\n\
No apodization.\n\
@item 'ud'\n\
User defined apodization.\n\
@item  'triangle'\n\
Triangle window.\n\
@item 'gauss'\n\
Gaussian (bell-shaped) window.\n\
@item 'raised'\n\
Raised cosine.\n\
@item 'simply'\n\
Simply supported.\n\
@item 'clamped'\n\
Clamped.\n\
@end table\n\
\n\
and the apod and win_par parameters are:\n\
\n\
@table @code\n\
@item apod\n\
Vector of apodiztion weights (used for the 'ud' option).\n\
@item win_par\n\
A scalar parameter for raised cosine and Gaussian apodization functions.\n\
@end table\n\
\n\
Error Handling: err_level;\n\
err_level is an optional text string parameter for controlling the error behavior, options are:\n\
\n\
@table @code\n\
@item 'ignore'\n\
An error is ignored (no error message is printed and the program is not stopped) but the err output \n\
argument is negative if an error occured.\n\
@item 'warn'\n\
An error message is printed but the program in not stopped (and err is negative).\n\
@item 'stop'\n\
An error message is printed and the program is stopped.\n\
@end table\n\
\n\
@code{dream_arr_cylind_f} is an oct-function that is a part of the DREAM Toolbox available at\n\
@url{http://www.signal.uu.se/Toolbox/dream/}.\n\
\n\
dream_arr_cylind_d is an oct-function that is a part of the DREAM Toolbox available at\n\
@url{http://www.signal.uu.se/Toolbox/dream/}.\n\
\n\
Copyright @copyright{} 2006-2008 Fredrik Lingvall.\n\
@seealso{dream_arr_cylind_d_p, dream_arr_cylind_f, dreamcylind_f}\n\
@end deftypefn")
{
  double *ro,*geom_par,*s_par,*m_par;
  double *steer_par;
  char   apod_met[50],foc_met[50],steer_met[50];
  int    buflen;
  double xo,yo,zo,a,b,R,dx,dy,dt;
  dream_idx_type    nt,no,n;
  double param=0,*delay,v,cp,alfa;
  int    isize;
  double *gx,*gy,*gz;
  int    ifoc=0;
  double focal=0, *ud_focal=NULL;
  int    ister=0;
  double theta=0,phi=0,*apod=NULL;
  int    iweight=0,iapo=0;
  double *h, *err_p;
  int    err_level=STOP, err=NONE, out_err = NONE , is_set = FALSE;
  char   err_str[50];
  sighandler_t   old_handler, old_handler_abrt, old_handler_keyint;
  octave_value_list oct_retval; 

  int nrhs = args.length ();
  
  // Check for proper number of arguments
  
  if (!((nrhs == 13) || (nrhs == 14))) {
    dream_err_msg("dream_arr_cylind_d requires 13 or 14 input arguments!");
  }
  else
    if (nlhs > 2) {
      dream_err_msg("Too many output arguments for dream_arr_cylind_d!");
    }
  
  //
  // Observation point.
  //
  
 // Check that arg 1 is a (number of observation points) x 3 matrix
  if (!mxGetN(0)==3) {
    error("Argument 1 must be a (number of observation points) x 3 matrix!");
    return oct_retval;
  }
  
  no = mxGetM(0); // Number of observation points.
  const Matrix tmp0 = args(0).matrix_value();
  ro = (double*) tmp0.fortran_vec();

  //
  // Transducer geometry
  //

  // Check that arg 2 is a 3 element vector
  if (!((mxGetM(1)==3 && mxGetN(1)==1) || (mxGetM(1)==1 && mxGetN(1)==3))) {
    error("Argument 2 must be a vector of length 3!");
    return oct_retval;
  }
  
  const Matrix tmp1 = args(1).matrix_value();
  geom_par = (double*) tmp1.fortran_vec();
  a = geom_par[0];		// x-width.
  b = geom_par[1];		// y-width.
  R = geom_par[2];		// Radius of the cylinder.

  //
  // Grid function (position vectors of the elements).
  //

  isize = (int) mxGetM(2); // Number of elementents in the array.
  if (mxGetN(2) !=3 )
    dream_err_msg("Argument 3  must a (number of array elements) x 3 matrix!");
  
  const Matrix tmp2 = args(2).matrix_value();
  gx = (double*) tmp2.fortran_vec(); // First column in the matrix.
  gy    = gx + isize;		// Second column in the matrix.
  gz    = gy + isize;		// Third column in the matrix.

  
  //
  // Temporal and spatial sampling parameters.
  //
  
  // Check that arg 4 is a 4 element vector
  if (!((mxGetM(3)==4 && mxGetN(3)==1) || (mxGetM(3)==1 && mxGetN(3)==4))) {
    error("Argument 4 must be a vector of length 4!");
    return oct_retval;
  }

  const Matrix tmp3 = args(3).matrix_value();
  s_par = (double*) tmp3.fortran_vec();
  dx    = s_par[0];		// Spatial x discretization size.
  dy    = s_par[1];		// Spatial dy iscretization size.
  dt    = s_par[2];		// Temporal discretization size (= 1/sampling freq).
  nt    = (dream_idx_type) s_par[3];	// Length of SIR.
  
  //
  // Start point of impulse response vector ([us]).
  //

  // Check that arg 5 is a scalar (or vector).
  if ( (mxGetM(4) * mxGetN(4) !=1) && ((mxGetM(4) * mxGetN(4)) != no)) {
    error("Argument 5 must be a scalar or a vector with a length equal to the number of observation points!");
    return oct_retval;
  }
  
  const Matrix tmp4 = args(4).matrix_value();
  delay = (double*) tmp4.fortran_vec();
  
  //
  // Material parameters
  //
  
  // Check that arg 6 is a 3 element vectora
  if (!((mxGetM(5)==3 && mxGetN(5)==1) || (mxGetM(5)==1 && mxGetN(5)==3))) {
    error("Argument 6 must be a vector of length 3!");
    return oct_retval;
  }
  
  const Matrix tmp5 = args(5).matrix_value();
  m_par = (double*) tmp5.fortran_vec();
  v     = m_par[0]; // Normal velocity of transducer surface.
  cp    = m_par[1]; // Sound speed.
  alfa  = m_par[2]; // Attenuation coefficient [dB/(cm MHz)].

  //  
  // Focusing parameters.
  //

  //  ifoc = 1 - no foc, 2 foc x ,3 foc y, 4 foc xy (del=fsqrt(x*x+y*y)), 5 focx+focy.

  if (nrhs >= 7) {

    if (!mxIsChar(6)) {
      error("Argument 7 must be a string");   
      return oct_retval;
    }

    std::string strin = args(6).string_value(); 
    buflen = strin.length();
    for ( n=0; n<=buflen; n++ ) {
      foc_met[n] = strin[n];
    }
    foc_met[buflen] = '\0';

    is_set = FALSE;
    
    if (!strcmp(foc_met,"off")) {
      ifoc = 1; 
      is_set = TRUE;
    }
    
    if (!strcmp(foc_met,"x")) {
      ifoc = 2; 
      is_set = TRUE;
    }
    
    if (!strcmp(foc_met,"y")) {
      ifoc = 3; 
      is_set = TRUE;
    }
    
    if (!strcmp(foc_met,"xy")) {
      ifoc = 4; 
      is_set = TRUE;
    }
    
    if (!strcmp(foc_met,"x+y")) {
      ifoc = 5; 
      is_set = TRUE;
    }
    
    if (!strcmp(foc_met,"ud")) {
      ifoc = 6; 
      is_set = TRUE;

      if (mxGetM(7) * mxGetN(7) != isize ) {
	error("The time delay vector (argument 8) for user defined ('ud') focusing\n") ;
	error("delays must have the same length as the number of array elements.!");
	return oct_retval;
      }
      const Matrix tmp4 = args(7).matrix_value();
      ud_focal = (double*) tmp4.fortran_vec();
    }
    else {
      
      // Check that arg 8 is a scalar.
      if (mxGetM(7) * mxGetN(7) !=1 ) {
	error("Argument 8  must be a scalar!");
      	return oct_retval;
      }
      // Focal point (in mm).
      const Matrix tmp4 = args(7).matrix_value();
      focal = (double) tmp4.fortran_vec()[0];
    }

    if (is_set == FALSE) {
      error("Unknown focusing method!");    
      return oct_retval;
    }
    
  } else
    ifoc = 1;
  
  
  //
  // Beam steering.
  //
  
  // Beam steering: ister = 1 - no steering, 2 steer ph=ax ,3 steer y ph=by, 4 steer xy ph=ax+by.
  
  if (nrhs >= 9) {
    
    if (!mxIsChar(8)) {
      error("Argument 9 must be a string");   
      return oct_retval;
    }
    
    std::string strin = args(8).string_value(); 
    buflen = strin.length();
    for ( n=0; n<=buflen; n++ ) {
      steer_met[n] = strin[n];
    }
    steer_met[buflen] = '\0';
    
    ister = 1;			// Default no steering
    is_set = FALSE;
    
    if (!strcmp(steer_met,"off")) {
      ister = 1;
      is_set = TRUE; 
    }

    if (!strcmp(steer_met,"x")) {
      ister = 2; 
      is_set = TRUE; 
    }
    
    if (!strcmp(steer_met,"y")) {
      ister = 3; 
      is_set = TRUE; 
    }

    if (!strcmp(steer_met,"xy")) {
      ister = 4; 
      is_set = TRUE; 
    }

    if (is_set == FALSE) {
      error("Unknown beamsteering method!");    
      return oct_retval;
    }

    // Check that arg 10 is a 2 element vector
    if (!((mxGetM(9)==2 && mxGetN(9)==1) || (mxGetM(9)==1 && mxGetN(9)==2))) {
      dream_err_msg("Argument 10 must be a vector of length 2!");
      return oct_retval;
    }
    
    const Matrix tmp5 = args(9).matrix_value();
    steer_par = (double*) tmp5.fortran_vec();
    theta  = steer_par[0];		// Angle in x-direction.
    phi    = steer_par[1];		// Angle in y-direction.
    
  } else
    ister = 1;
  
  //
  // Apodization.
  //

  // iweight = 1 - no weighting, 2  weighting.
  // iapo = 0 - user defined, 1 traingle, 2 Gauss, 3 raised cosine, 4 simply supported, 5 clamped. 

  if (nrhs >= 11) {
    
    if (!mxIsChar(10)) {
      error("Argument 11 must be a string");    
      return oct_retval;
    }

    std::string strin = args(10).string_value(); 
    buflen = strin.length();
    for ( n=0; n<=buflen; n++ ) {
      apod_met[n] = strin[n];
    }
    apod_met[buflen] = '\0';
    
    iweight = 1;			// default off. 
    is_set = FALSE;
    
    if (!strcmp(apod_met,"off")) {
      iweight = 1; 
      is_set = TRUE; 
    }
    
    if (!strcmp(apod_met,"ud")) {
      iweight = 2; 
      iapo = 0; 
      is_set = TRUE; 

      // Vector of apodization weights.    
      if (mxGetM(11) * mxGetN(11) != isize) {
	error("The length of argument 12 (apodization vector) must be the same as the number of array elements!");
	return oct_retval;
      }
      
      const Matrix tmp6 = args(11).matrix_value();
      apod = (double*) tmp6.fortran_vec();
    }
    
    if (!strcmp(apod_met,"triangle")) {
      iweight = 2; 
      iapo = 1; 
      is_set = TRUE; 
    }
    
    if (!strcmp(apod_met,"gauss")) {
      iweight = 2; 
      iapo = 2; 
      is_set = TRUE; 
    }
    
    if (!strcmp(apod_met,"raised")) {
      iweight = 2; 
      iapo = 3; 
      is_set = TRUE; 
    }
    
    if (!strcmp(apod_met,"simply")) {
      iweight = 2; 
      iapo = 4; 
      is_set = TRUE; 
    }
    
    if (!strcmp(apod_met,"clamped")) {
      iweight = 2; 
      iapo = 5; 
      is_set = TRUE; 
    }

    if (is_set == FALSE) {
      error("Unknown apodization!");
      return oct_retval;
    }

    // Parameter for raised cos and Gaussian apodization functions.
    if (mxGetM(12) * mxGetN(12) !=1 ) {
      error("Argument 13 must be a scalar");
      return oct_retval;
    }

    const Matrix tmp7 = args(12).matrix_value();
    param = (double) tmp7.fortran_vec()[0];
  }
  else 
    iweight = 1;   


  // Error reporting.
  if (nrhs == 14) { 

    if (!mxIsChar(13)) {
      dream_err_msg("Argument 14 must be a string");   
      return oct_retval;
    }

    std::string strin = args(13).string_value(); 
    buflen = strin.length();
    for ( n=0; n<=buflen; n++ ) {
      err_str[n] = strin[n];
    }
    err_str[buflen] = '\0';
    
    is_set = FALSE;

    if (!strcmp(err_str,"ignore")) {
      err_level = IGNORE; 
      is_set = TRUE;
    }

    if (!strcmp(err_str,"warn")) {
      err_level = WARN; 
      is_set = TRUE;
    }

    if (!strcmp(err_str,"stop")) {
      err_level = STOP; 
      is_set = TRUE;
    }
    
    if (is_set == FALSE) {
      error("Unknown error level!");
      return oct_retval;
    }
  }
  else
    err_level = STOP; // Default.

 // Create an output matrix for the impulse response
  Matrix h_mat(nt, no);
  h = h_mat.fortran_vec();

  //
  // Register signal handlers.
  //

  if ((old_handler = signal(SIGTERM, &sighandler)) == SIG_ERR) {
    printf("Couldn't register SIGTERM  signal handler.\n");
  }

  if (( old_handler_abrt=signal(SIGABRT, &sighandler)) == SIG_ERR) {
    printf("Couldn't register SIGABRT signal handler.\n");
  }
  
  if (( old_handler_keyint=signal(SIGINT, &sighandler)) == SIG_ERR) {
    printf("Couldn't register SIGINT signal handler.\n");
  }

  //  
  // Call the DREAM subroutine.
  //
  
  running = TRUE;

#ifdef USE_FFTW
  if (alfa != (double) 0.0)
    att_init(nt,1);
#endif

  if (ifoc != 6) { 
    
    if (mxGetM(4) * mxGetN(4) == 1) {
      for (n=0; n<no; n++) {
	xo = ro[n]; 
	yo = ro[n+1*no]; 
	zo = ro[n+2*no]; 
	
	err = dream_arr_cylind_d(xo,yo,zo,a,b,R,dx,dy,dt,nt,delay[0],v,cp,alfa,isize,gx,gy,gz,
			     ifoc,focal,ister,theta,phi,apod,iweight,iapo,param,&h[n*nt],err_level);
	if (err != NONE) {
	  out_err = err;
	  if (err == STOP) {
	    error("");
	    return oct_retval;
	  }
	}
	
	if (!running) {
	  break; // CTRL-C pressed.
	}
 
      }
    } else {
      for (n=0; n<no; n++) {
	xo = ro[n]; 
	yo = ro[n+1*no]; 
	zo = ro[n+2*no]; 
	
	err = dream_arr_cylind_d(xo,yo,zo,a,b,R,dx,dy,dt,nt,delay[n],v,cp,alfa,isize,gx,gy,gz,
			     ifoc,focal,ister,theta,phi,apod,iweight,iapo,param,&h[n*nt],err_level);
	if (err != NONE) {
	  out_err = err;
	  if (err == STOP) {
	    error("");
	    return oct_retval;
	  }
	}
	
      }
    }
  }
  else { // User defined focusing.
    
    if (mxGetM(4) * mxGetN(4) == 1) {
      for (n=0; n<no; n++) {
	xo = ro[n]; 
	yo = ro[n+1*no]; 
	zo = ro[n+2*no]; 
	
	err = dream_arr_cylind_udd(xo,yo,zo,a,b,R,dx,dy,dt,nt,delay[0],v,cp,alfa,isize,gx,gy,gz,
				ifoc,ud_focal,ister,theta,phi,apod,iweight,iapo,param,&h[n*nt],err_level);
	if (err != NONE) {
	  out_err = err;
	  if (err == STOP) {
	    error("");
	    return oct_retval;
	  }
	}
	
	if (!running) {
	  break; // CTRL-C pressed.
	}

      }
    } else {
      for (n=0; n<no; n++) {
	xo = ro[n]; 
	yo = ro[n+1*no]; 
	zo = ro[n+2*no]; 
	
	err = dream_arr_cylind_udd(xo,yo,zo,a,b,R,dx,dy,dt,nt,delay[n],v,cp,alfa,isize,gx,gy,gz,
				   ifoc,ud_focal,ister,theta,phi,apod,iweight,iapo,param,&h[n*nt],err_level);
	if (err != NONE) {
	  out_err = err;
	  if (err == STOP) {
	    error("");
	    return oct_retval;
	  }
	}
	
	if (!running) {
	  break; // CTRL-C pressed.
	}
	
      }
    }
  }

  //
  // Restore old signal handlers.
  //

  if (signal(SIGTERM, old_handler) == SIG_ERR) {
    printf("Couldn't register old SIGTERM signal handler.\n");
  }
   
  if (signal(SIGABRT,  old_handler_abrt) == SIG_ERR) {
    printf("Couldn't register old SIGABRT signal handler.\n");
  }
  
  if (signal(SIGINT, old_handler_keyint) == SIG_ERR) {
    printf("Couldn't register old SIGINT signal handler.\n");
  }

#ifdef USE_FFTW
  if (alfa != (double) 0.0)
    att_close();
#endif 

  if (!running) {
    error("CTRL-C pressed!\n"); // Bail out.
    return oct_retval; 
  }

  oct_retval.append(h_mat);
  
  // Return error.
  if (nlhs == 2) {
    Matrix err_mat(nt, no);
    err_p = err_mat.fortran_vec();
    err_p[0] = (double) out_err;
    oct_retval.append(err_mat);
  }
  
  return oct_retval;
}
      
