
####################################################################
#
# Script for cross compiling fftw for Windows 32 bit. You need
# to install the MinGW (i686-mingw32) cross compiler tool chain
# to run this.   
#
# Copyright (C) 2009 Fredrik Lingvall
#
####################################################################

# $Revision: 598 $ $Date: 2009-11-08 14:55:25 +0100 (Sun, 08 Nov 2009) $ $LastChangedBy: dream $

CONF_FLAGS="CC=i686-mingw32-gcc --host=i686-mingw32 --with-gcc-arch=pentium4 --enable-portable-binary --with-our-malloc16 --with-windows-f77-mangling --enable-shared --disable-static --enable-threads --with-combined-threads"

if [ -d fftw-3.2.2 ]; then
    cd fftw-3.2.2
else
    tar xzf fftw-3.2.2.tar.gz
    cd fftw-3.2.2
fi

rm -fr mingw

rm -fr mkdir double-mingw
mkdir double-mingw
cd double-mingw
../configure --prefix=`pwd`/../mingw ${CONF_FLAGS} \
    --enable-sse2 && make -j4 && make install
cp -f tests/.libs/bench.exe `pwd`/../mingw/bin/bench.exe
cd ..

rm -fr single-mingw
mkdir single-mingw
cd single-mingw
../configure --prefix=`pwd`/../mingw ${CONF_FLAGS} \
    --enable-sse --enable-float && make -j4 && make install
cp -f tests/.libs/bench.exe `pwd`/../mingw/bin/benchf.exe
cd ..

rm -fr ldouble-mingw
mkdir ldouble-mingw
cd ldouble-mingw 
../configure --prefix=`pwd`/../mingw ${CONF_FLAGS} \
    --enable-long-double && make -j4 && make install
cp -f tests/.libs/bench.exe `pwd`/../mingw/bin/benchl.exe
cd ..

cd mingw/bin
for dll in *.dll; do
    def=`basename $dll .dll`.def
    echo "LIBRARY $dll" > $def
    echo EXPORTS >> $def
    i686-mingw32-nm $dll | grep ' T _' | sed 's/.* T _//' | grep fftw >> $def
done
cd ../..

perl -pi -e 's,^ * #define FFTW_DLL,*/\n#define FFTW_DLL\n/*,' mingw/include/fftw3.h

cat > README-WINDOWS <<EOF
This .zip archive contains DLL libraries and the associated header (.h)
and module-definition (.def) files of FFTW compiled for Win32.  It
also contains the corresponding bench.exe test/benchmark programs
and wisdom utilities.

There are three libraries: single precision (float), double precision,
and extended precision (long double).  To use the third library,
your compiler must have sizeof(long double) == 12.

In order to link to these .dll files from Visual C++, you need to
create .lib "import libraries" for them, and can do so with the "lib"
command that comes with VC++.  In particular, run:
     lib /machine:i386 /def:libfftw3-3.def
     lib /machine:i386 /def:libfftw3f-3.def
     lib /machine:i386 /def:libfftw3l-3.def

The single- and double-precision libraries use SSE and SSE2, respectively,
but should also work on older processors (the library checks at runtime
to see whether SSE/SSE2 is supported and disables the relevant code if not).

They were compiled by the GNU C compiler for MinGW, specifically:
EOF
i686-mingw32-gcc --version |head -1 >> README-WINDOWS
cp -f tests/README README-bench

fftw_vers=`grep PACKAGE_VERSION double-mingw/config.h |cut -d" " -f3 |tr -d \"`
zip=fftw-${fftw_vers}-dll.zip
rm -f $zip
zip -vj $zip mingw/bin/*.dll mingw/bin/*.exe
zip -vjgl $zip mingw/bin/*.def mingw/include/* README COPYING COPYRIGHT NEWS README-WINDOWS README-bench

mv fftw-${fftw_vers}-dll.zip ../
cd ..

# Install the lib in the dll folder.
cd dll
unzip ../fftw-${fftw_vers}-dll.zip

# Note: you must create the lib files to 
# use this lib with MSCV. See the file:
#
#  make_fftw_lib_file_msvc.bat
#
# in the dll folder.

cd ..
